/**
 * A library for compiling and running TI-Basic, built for node.
 *
 * @packageDocumentation
 */

/// <reference types="jquery" />

/**
 * @alpha
 */
declare const ANS = "ans";

/**
 * @alpha
 */
declare type Assignable = Variable | ListIndex;

/**
 * @alpha
 */
declare const AssignmentStatement = "assign";

/**
 * @alpha
 */
declare type AssignmentStatement = {
    type: typeof AssignmentStatement;
    source?: string;
    value: ValueExpression;
    assignable: Assignable;
};

/**
 * @alpha
 */
declare const BINARY = "binary";

/**
 * @alpha
 */
declare type BinaryExpression = {
    type: typeof BINARY;
    operator: string;
    left: ValueExpression;
    right: ValueExpression;
};

/**
 * @alpha
 */
declare const ClrHome = "ClrHome";

/**
 * @alpha
 */
declare type ClrHome = {
    type: typeof ClrHome;
    source?: string;
};

/**
 * @alpha
 */
declare const ClrTable = "ClrTable";

/**
 * @alpha
 */
declare type ClrTable = {
    type: typeof ClrTable;
    source?: string;
};

/**
 * @alpha
 */
declare type CtlStatement = IfStatement | ThenStatement | ElseStatement | ForLoop | WhileLoop | RepeatLoop | EndStatement | PauseStatement | LabelStatement | GotoStatement | IncrementSkip | DecrementSkip | MenuStatement | ProgramStatement | ReturnStatement | StopStatement | DelVarStatement | GraphStyleStatement | OpenLibStatement | ExecLibStatement;

/**
 * @alpha
 */
declare const DecrementSkip = "DecrementSkip";

/**
 * @alpha
 */
declare type DecrementSkip = {
    type: typeof DecrementSkip;
    source?: string;
    variable: Variable | null;
    end: ValueExpression | null;
};

/**
 * @alpha
 */
declare const DelVarStatement = "DelVarStatement";

/**
 * @alpha
 */
declare type DelVarStatement = {
    type: typeof DelVarStatement;
    source?: string;
    variable: Variable | null;
};

/**
 * @alpha
 */
declare const DispGraph = "DispGraph";

/**
 * @alpha
 */
declare type DispGraph = {
    type: typeof DispGraph;
    source?: string;
};

/**
 * @alpha
 */
declare const Display = "display";

/**
 * @alpha
 */
declare type Display = {
    type: typeof Display;
    source?: string;
    value: ValueExpression | null;
};

/**
 * @alpha
 */
declare const DispTable = "DispTable";

/**
 * @alpha
 */
declare type DispTable = {
    type: typeof DispTable;
    source?: string;
};

/**
 * @alpha
 */
declare const ElseStatement = "ElseStatement";

/**
 * @alpha
 */
declare type ElseStatement = {
    type: typeof ElseStatement;
    source?: string;
    extra: boolean | null;
};

/**
 * @alpha
 */
declare const EmptyStatement = "empty";

/**
 * @alpha
 */
declare type EmptyStatement = {
    type: typeof EmptyStatement;
    source?: string;
};

/**
 * @alpha
 */
declare const EndStatement = "EndStatement";

/**
 * @alpha
 */
declare type EndStatement = {
    type: typeof EndStatement;
    source?: string;
    extra: boolean | null;
};

/**
 * @alpha
 */
export declare function exec(source: string, callback: (output: string) => void): void;

/**
 * @alpha
 */
declare const ExecLibStatement = "ExecLibStatement";

/**
 * @alpha
 */
declare type ExecLibStatement = {
    type: typeof ExecLibStatement;
    source?: string;
    name: string;
};

/**
 * @alpha
 */
declare const ForLoop = "ForLoop";

/**
 * @alpha
 */
declare type ForLoop = {
    type: typeof ForLoop;
    source?: string;
    variable: Variable;
    start: ValueExpression | null;
    end: ValueExpression | null;
    step: ValueExpression | null;
    args: boolean;
    extra: boolean | null;
};

/**
 * @alpha
 */
declare const Get = "Get";

/**
 * @alpha
 */
declare type Get = {
    type: typeof Get;
    source?: string;
    variable: Variable | null;
};

/**
 * @alpha
 */
declare const GetCalc = "GetCalc";

/**
 * @alpha
 */
declare type GetCalc = {
    type: typeof GetCalc;
    source?: string;
    variable: Variable | null;
    portflag: ValueExpression | null;
};

/**
 * @alpha
 */
declare const GetKey = "GetKey";

/**
 * @alpha
 */
declare const GotoStatement = "GotoStatement";

/**
 * @alpha
 */
declare type GotoStatement = {
    type: typeof GotoStatement;
    source?: string;
    location: string;
};

/**
 * @alpha
 */
declare const GraphStyleStatement = "GraphStyleStatement";

/**
 * @alpha
 */
declare type GraphStyleStatement = {
    type: typeof GraphStyleStatement;
    source?: string;
    equation: ValueExpression | null;
    style: ValueExpression | null;
};

/**
 * @alpha
 */
declare const IfStatement = "IfStatement";

/**
 * @alpha
 */
declare type IfStatement = {
    type: typeof IfStatement;
    source?: string;
    value: ValueExpression | null;
    extra: boolean | null;
};

/**
 * @alpha
 */
declare const IncrementSkip = "IncrementSkip";

/**
 * @alpha
 */
declare type IncrementSkip = {
    type: typeof IncrementSkip;
    source?: string;
    variable: Variable | null;
    end: ValueExpression | null;
};

/**
 * @alpha
 */
declare const Input = "Input";

/**
 * @alpha
 */
declare type Input = {
    type: typeof Input;
    source?: string;
    text: ValueExpression | null;
    variable: Variable | null;
};

/**
 * @alpha
 */
declare type IoStatement = Input | Prompt | Display | DispGraph | DispTable | Output | ClrHome | ClrTable | GetCalc | Get | Send;

/**
 * @alpha
 */
declare const LabelStatement = "LabelStatement";

/**
 * @alpha
 */
declare type LabelStatement = {
    type: typeof LabelStatement;
    source?: string;
    location: string;
};

/**
 * @alpha
 */
declare const lib = "lib";

/**
 * @alpha
 */
declare const LIST = "list";

/**
 * @alpha
 */
declare const LISTINDEX = "listIndex";

/**
 * @alpha
 */
declare type ListIndex = {
    type: 'listIndex';
    list: ListVariable;
    index: ValueExpression;
};

/**
 * @alpha
 */
declare type ListLiteral = {
    type: typeof LIST;
    resolved: false;
    elements: Array<ValueExpression>;
};

/**
 * @alpha
 */
declare type ListResolved = {
    type: typeof LIST;
    resolved: true;
    elements: Array<NumberResolved>;
};

/**
 * @alpha
 */
declare const LISTVARIABLE = "listVariable";

/**
 * @alpha
 */
declare type ListVariable = {
    type: 'listVariable';
    name: string;
    custom: boolean;
};

/**
 * @alpha
 */
declare const ListVariablePrefix = "List";

/**
 * @alpha
 */
declare type MenuChoice = {
    option: TiString;
    location: string;
};

/**
 * @alpha
 */
declare const MenuStatement = "MenuStatement";

/**
 * @alpha
 */
declare type MenuStatement = {
    type: typeof MenuStatement;
    source?: string;
    title: ValueExpression | null;
    choices: Array<MenuChoice>;
};

/**
 * @alpha
 */
declare const NUMBER = "number";

/**
 * @alpha
 */
declare type NumberLiteral = {
    type: typeof NUMBER;
    resolved: false;
    integer?: number;
    fraction?: number | null;
    exponent?: number | null;
};

/**
 * @alpha
 */
declare type NumberResolved = {
    type: typeof NUMBER;
    resolved: true;
    float: number;
};

/**
 * @alpha
 */
declare type NumericVariable = {
    type: 'variable';
    name: string;
};

/**
 * @alpha
 */
export declare function on(type: string, listener: () => void): void;

/**
 * @alpha
 */
declare const OpenLibStatement = "OpenLibStatement";

/**
 * @alpha
 */
declare type OpenLibStatement = {
    type: typeof OpenLibStatement;
    source?: string;
    name: string;
};

/**
 * @alpha
 */
declare const Output = "Output";

/**
 * @alpha
 */
declare type Output = {
    type: typeof Output;
    source?: string;
    row: ValueExpression | null;
    column: ValueExpression | null;
    value: ValueExpression | null;
};

/**
 * @alpha
 */
export declare function parse(source: string, options?: ParseOptions): types.Statement[];

/**
 * @alpha
 */
export declare type ParseOptions = {
    sourceMap?: string;
};

/**
 * @alpha
 */
declare const PauseStatement = "PauseStatement";

/**
 * @alpha
 */
declare type PauseStatement = {
    type: typeof PauseStatement;
    source?: string;
};

/**
 * @alpha
 */
declare const ProgramStatement = "ProgramStatement";

/**
 * @alpha
 */
declare type ProgramStatement = {
    type: typeof ProgramStatement;
    source?: string;
    name: string;
};

/**
 * @alpha
 */
declare const Prompt = "Prompt";

/**
 * @alpha
 */
declare type Prompt = {
    type: typeof Prompt;
    source?: string;
    variable: Variable | null;
};

/**
 * @alpha
 */
declare const RepeatLoop = "RepeatLoop";

/**
 * @alpha
 */
declare type RepeatLoop = {
    type: typeof RepeatLoop;
    source?: string;
    value: ValueExpression | null;
    extra: boolean | null;
};

/**
 * @alpha
 */
declare const ReturnStatement = "ReturnStatement";

/**
 * @alpha
 */
declare type ReturnStatement = {
    type: typeof ReturnStatement;
    source?: string;
};

/**
 * @alpha
 */
export declare function run(lines: Array<types.Statement>, options?: RunOptions): {
    getStatus: () => string;
    isActive: () => boolean;
    stop: () => void;
};

/**
 * @alpha
 */
export declare type RunOptions = {
    source?: string | Array<string>;
    frequencyMs?: number;
    outputCallback?: (value: string, newline: boolean) => void;
    elem?: JQuery<HTMLElement>;
    debug?: boolean;
    callback?: (status: string) => void;
    input?: JQuery<HTMLElement>;
    stdin?: string;
    includeErrors?: boolean;
    includeLibErrors?: boolean;
    includeLineNumbers?: boolean;
    includeSource?: boolean;
};

/**
 * @alpha
 */
declare const Send = "Send";

/**
 * @alpha
 */
declare type Send = {
    type: typeof Send;
    source?: string;
    variable: Variable | null;
};

/**
 * @alpha
 */
declare type Statement = EmptyStatement | SyntaxError_2 | AssignmentStatement | CtlStatement | IoStatement | ValueStatement;

/**
 * @alpha
 */
declare const StopStatement = "StopStatement";

/**
 * @alpha
 */
declare type StopStatement = {
    type: typeof StopStatement;
    source?: string;
};

/**
 * @alpha
 */
declare const STRING = "string";

/**
 * @alpha
 */
declare const STRINGVARIABLE = "stringVariable";

/**
 * @alpha
 */
declare type StringVariable = {
    type: 'stringVariable';
    name: string;
};

/**
 * @alpha
 */
declare const SyntaxError_2 = "SyntaxError";

/**
 * @alpha
 */
declare type SyntaxError_2 = {
    type: typeof SyntaxError_2;
    source?: string;
};

/**
 * @alpha
 */
declare const ThenStatement = "ThenStatement";

/**
 * @alpha
 */
declare type ThenStatement = {
    type: typeof ThenStatement;
    source?: string;
    extra: boolean | null;
};

/**
 * @alpha
 */
declare const ti = "ti";

/**
 * @alpha
 */
declare type TiList = ListLiteral | ListResolved;

/**
 * @alpha
 */
declare type TiNumber = NumberLiteral | NumberResolved;

/**
 * @alpha
 */
declare type TiString = {
    type: typeof STRING;
    chars: string;
};

declare namespace types {
    export {
        ti,
        lib,
        SyntaxError_2 as SyntaxError,
        NUMBER,
        LIST,
        STRING,
        ListVariablePrefix,
        NumberLiteral,
        NumberResolved,
        TiNumber,
        TiString,
        ListLiteral,
        ListResolved,
        TiList,
        ValueResolved,
        ANS,
        GetKey,
        VARIABLE,
        STRINGVARIABLE,
        LISTVARIABLE,
        LISTINDEX,
        NumericVariable,
        StringVariable,
        ListVariable,
        ListIndex,
        Variable,
        Assignable,
        BINARY,
        UNARY,
        BinaryExpression,
        UnaryExpression,
        ValueExpression,
        EmptyStatement,
        ValueStatement,
        AssignmentStatement,
        IfStatement,
        ThenStatement,
        ElseStatement,
        ForLoop,
        WhileLoop,
        RepeatLoop,
        EndStatement,
        PauseStatement,
        LabelStatement,
        GotoStatement,
        IncrementSkip,
        DecrementSkip,
        MenuStatement,
        ProgramStatement,
        ReturnStatement,
        StopStatement,
        DelVarStatement,
        GraphStyleStatement,
        OpenLibStatement,
        ExecLibStatement,
        MenuChoice,
        CtlStatement,
        Input,
        Prompt,
        Display,
        DispGraph,
        DispTable,
        Output,
        ClrHome,
        ClrTable,
        GetCalc,
        Get,
        Send,
        IoStatement,
        Statement
    }
}
export { types }

/**
 * @alpha
 */
declare const UNARY = "unary";

/**
 * @alpha
 */
declare type UnaryExpression = {
    type: typeof UNARY;
    operator: string;
    argument: ValueExpression;
};

/**
 * @alpha
 */
declare type ValueExpression = TiNumber | TiString | TiList | Assignable | BinaryExpression | UnaryExpression;

/**
 * @alpha
 */
declare type ValueResolved = NumberResolved | TiString | ListResolved;

/**
 * @alpha
 */
declare const ValueStatement = "value";

/**
 * @alpha
 */
declare type ValueStatement = {
    type: typeof ValueStatement;
    value: ValueExpression;
    source?: string;
};

/**
 * @alpha
 */
declare const VARIABLE = "variable";

/**
 * @alpha
 */
declare type Variable = NumericVariable | StringVariable | ListVariable;

/**
 * @alpha
 */
declare const WhileLoop = "WhileLoop";

/**
 * @alpha
 */
declare type WhileLoop = {
    type: typeof WhileLoop;
    source?: string;
    value: ValueExpression | null;
    extra: boolean | null;
};

export { }
